package com.retail_store.order_management.service;

import com.retail_store.order_management.constants.Category;
import com.retail_store.order_management.constants.Color;
import com.retail_store.order_management.constants.Gender;
import com.retail_store.order_management.constants.Size;
import com.retail_store.order_management.model.Order;
import com.retail_store.order_management.model.OrderedProduct;
import com.retail_store.order_management.model.Product;

import java.sql.Date;
import java.sql.*;
import java.time.LocalDate;
import java.time.LocalTime;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;

import static com.retail_store.order_management.utils.Database.getConnection;
import static com.retail_store.order_management.utils.Utility.*;

/**
 * Class RetailStoreService
 * This class implements from previous interfaces RetailStore and ProductFilter so
 * that all the unimplemented methods should be implemented here and given there logic inside
 */
public class RetailStoreService implements RetailStore, ProductFilter {

    public boolean addProducts() {
        boolean productAdded = false;
        String id = productIdGenerator();
        getLogger().info("Enter Product details \n Enter Product Name");
        String name = getScanner().nextLine();
        Category category = getCategory(true, "Enter Product Category");
        if (category == null) {
            return false;
        }
        Color color = getColor(true, "Select color of the product");
        if (category == null) {
            return false;
        }
        Gender gender = getGender(true, "Product is ideal for");
        if (category == null) {
            return false;
        }
        double price = getDouble(true, "Enter price of the product");
        Map<Size, Integer> size = getProductSizes();
        Connection connection = getConnection();
        try {
            PreparedStatement preparedStatement = connection.prepareStatement("insert into products values(?,?,?,?,?,?,?,?,?,?,?)");
            preparedStatement.setString(1, id);
            preparedStatement.setString(2, name);
            preparedStatement.setString(3, category.toString());
            preparedStatement.setString(4, color.toString());
            preparedStatement.setDouble(5, price);
            preparedStatement.setString(6, gender.getGender());
            preparedStatement.setInt(7, size.get(Size.S));
            preparedStatement.setInt(8, size.get(Size.M));
            preparedStatement.setInt(9, size.get(Size.L));
            preparedStatement.setInt(10, size.get(Size.XL));
            preparedStatement.setInt(11, size.get(Size.XXL));
            productAdded = preparedStatement.executeUpdate() > 0;
            if (productAdded) {
                getLogger().info("Your product with name : " + name + " is added successfully");
            } else {
                getLogger().info("Your product is not added successfully");
                productAdded = addProducts();
            }
        } catch (SQLException e) {
            getScanner().reset();
            getLogger().warning(e.getMessage());
            getLogger().warning("Your product is not added successfully");
            productAdded = addProducts();
        }
        return productAdded;
    }

    public boolean placeOrders() {
        Connection connection = getConnection();
        int confirmOrder;
        String orderMessage;
        try {
            connection.setAutoCommit(false);

            List<OrderedProduct> orderedProductList = createOrder(connection);
            if (orderedProductList == null || orderedProductList.isEmpty()) {
                getLogger().info("Since your order list is empty, unable to place order\n To make an order please enter 1\n Press any number to exit");
                int option = getInt();
                if (option == 1) {
                    return placeOrders();
                } else {
                    getScanner().reset();
                    return false;
                }
            }
            String orderId = orderIdGenerator();
            getLogger().info("Please enter Customer name");
            String name = getScanner().nextLine();
            getLogger().info("Please enter Phone number");
            String phone = phoneNumberValidator();
            Date date = Date.valueOf(LocalDate.now());
            Time time = Time.valueOf(LocalTime.now());
            double bill = orderedProductList.stream().map(product -> product.getPrice() * product.getQuantity()).reduce(0.00, (totalBill, productBill) -> totalBill + productBill);
            orderMessage = "Order ID: " + orderId + "\nName: " + name + "\nProducts: " + orderedProductList.stream().map(product -> product.getName() + "\t-\tPrice: " + product.getPrice() + "\t-\tQuantity: " + product.getQuantity() + "\t-\tCost: " + product.getPrice() * product.getQuantity()).reduce("", (products, product) -> products + "\n" + product) + "\nTotal Bill : " + bill;
            PreparedStatement preparedStatement = connection.prepareStatement("insert into orders values(?,?,?,?,?,?)");
            preparedStatement.setString(1, orderId);
            preparedStatement.setString(2, name);
            preparedStatement.setString(3, phone);
            preparedStatement.setDate(4, date);
            preparedStatement.setTime(5, time);
            preparedStatement.setDouble(6, bill);
            if (preparedStatement.executeUpdate() > 0) {
                preparedStatement = connection.prepareStatement("insert into ordered_products values (?,?,?,?,?,?,?,?,?)");
                for (OrderedProduct product : orderedProductList) {
                    preparedStatement.setString(1, product.getId());
                    preparedStatement.setString(2, product.getName());
                    preparedStatement.setString(3, product.getCategory().toString());
                    preparedStatement.setString(4, product.getColor().toString());
                    preparedStatement.setDouble(5, product.getPrice());
                    preparedStatement.setString(6, product.getIdealFor().getGender());
                    preparedStatement.setString(7, product.getSize().toString());
                    preparedStatement.setInt(8, product.getQuantity());
                    preparedStatement.setString(9, orderId);
                    preparedStatement.addBatch();
                }
                preparedStatement.executeBatch();
            }
            getLogger().info("to confirm the order enter 1\n to cancel enter any number");
            confirmOrder = getScanner().nextInt();
            switch (confirmOrder) {
                case 1:
                    getLogger().info("Your order is successfully placed.");
                    getLogger().info(orderMessage);
                    connection.commit();
                    break;
                default:
                    getLogger().info("Your order is not confirmed");
                    connection.rollback();
            }
            return true;
        } catch (Exception e) {
            getLogger().warning(e.getMessage());
            try {
                connection.rollback();
            } catch (Exception ex) {
                getLogger().warning(ex.getMessage());
            }
            return true;
        }
    }

    @Override
    public List<OrderedProduct> createOrder(Connection connection) {
        List<OrderedProduct> cart = new ArrayList<>();
        boolean orderBoolean = true;
        while (orderBoolean) {
            getLogger().info("********************GAP********************* " + "\n \t (1) Men  \t \t \t \t (2) Women\n-----------------------------\nto exit enter 0\n-----------------------------\n");
            Gender gender = getGender(false, "Please select...");
            if (gender == Gender.EXIT) {
                cart = null;
                break;
            }
            boolean genderBoolean = true;
            while (genderBoolean) {
                Map<Integer, Category> categoryMenu = getCategoriesByGender(gender);
                getLogger().info("\n-----------------------------\nSelect the category\n-----------------------------\nTo change the gender please enter 0");
                int categoryMenuNumber = getInt();
                Category category = categoryMenu.get(categoryMenuNumber);
                if (category == null) {
                    break;
                }
                boolean categoryBoolean = true;
                while (categoryBoolean) {
                    Map<Integer, Product> productsByGenderAndCategory = getProductsByGenderAndCategory(gender, category);
                    getLogger().info("\n-----------------------------\nSelect the product\n-----------------------------\nTo change the category please enter 0");
                    int productMenuNumber = getInt();
                    Product product = productsByGenderAndCategory.get(productMenuNumber);
                    if (product == null) {
                        break;
                    }
                    if (!addProductToCart(cart, product, connection)) {
                        continue;
                    } else {
                        getLogger().info("\n-----------------------------\nYour product : " + product.getName() + " is added to your cart.\n-----------------------------\n");
                    }
                    boolean menuFlag = true;
                    getLogger().info("\n-----------------------------\nPlease enter 1 - to continue the shopping" +
                            "\n-----------------------------\nEnter 2 to place the order" +
                            "\n-----------------------------\nEnter 3 to go back");
                    while (menuFlag) {
                        int order = getInt();
                        switch (order) {
                            case 1:
                                categoryBoolean = false;
                                genderBoolean = false;
                                menuFlag = false;
                                break;
                            case 2:
                                categoryBoolean = false;
                                genderBoolean = false;
                                menuFlag = false;
                                orderBoolean = false;
                                break;
                            case 3:
                                cart = null;
                                categoryBoolean = false;
                                genderBoolean = false;
                                menuFlag = false;
                                orderBoolean = false;
                                break;
                            default:
                                getLogger().info("\n-----------------------------\nPlease enter valid option\n-----------------------------\n");
                                createOrder(connection);
                        }
                    }
                }
            }
        }
        return cart;
    }

    @Override
    public synchronized boolean addProductToCart(Collection<OrderedProduct> cart, Product product, Connection connection) {
        if (product.outOfStock()) {
            getLogger().info("\n-----------------------------\nSorry to say this. Currently this product was out of stock.\n-----------------------------\n");
            return false;
        }
        getLogger().info(product.showSizes());
        Size size = getSize(true, "Please Select Size");
        if (size == null) {
            return false;
        }
        getLogger().info("\n-----------------------------\nPlease enter quantity required\n-----------------------------\n");
        int quantity = getInt();
        if (quantity <= 0) {
            getLogger().info("\n-----------------------------\nPlease enter atleast 1 quantity.\n-----------------------------\n");
            return false;
        }
        int availableQuantity = product.getSize().get(size);
        if (quantity <= availableQuantity) {
            int remaining = availableQuantity - quantity;
            try {
                PreparedStatement preparedStatement = connection.prepareStatement("update gap.products set " + size.getName() + "=? where id =?");
                preparedStatement.setInt(1, remaining);
                preparedStatement.setString(2, product.getId());
                preparedStatement.executeUpdate();
            } catch (SQLException e) {
                getLogger().warning("\n-----------------------------\nUnable to add product to cart\n-----------------------------\n");
                getLogger().warning(e.getMessage());
                return addProductToCart(cart, product, connection);
            }
            return cart.add(new OrderedProduct(product.getId(), product.getName(), product.getCategory(), product.getColor(), product.getPrice(), product.getIdealFor(), size, quantity));
        } else {
            getLogger().warning("\n-----------------------------\nplease enter quantity less or equal to the stock available\n-----------------------------\n");
            return addProductToCart(cart, product, connection);
        }
    }

    @Override
    public Map<Integer, Category> getCategoriesByGender(Gender gender) {
        Map<Integer, Category> categoryMap = new LinkedHashMap<>();
        AtomicInteger number = new AtomicInteger(0);
        Connection connection = getConnection();
        try {
            PreparedStatement preparedStatement = connection.prepareStatement("select category from gap.products where gender=? group by category");
            preparedStatement.setString(1, gender.getGender());
            ResultSet resultSet = preparedStatement.executeQuery();
            while (resultSet.next()) {
                categoryMap.put(number.incrementAndGet(), Category.valueOf(resultSet.getString(1)));
            }
        } catch (SQLException e) {
            getLogger().warning(e.getMessage());
            getLogger().warning("\n-----------------------------\ngot error while fetching the gender. please try again\n-----------------------------\n");
            return getCategoriesByGender(gender);
        }
        if (categoryMap.isEmpty()) {
            getLogger().info("\n-----------------------------\nCategories not found in " + gender + "\n-----------------------------\n");
        } else {
            String menu = categoryMap.entrySet().stream().map(entry -> entry.getKey() + ". " + entry.getValue()).reduce("", (categories, category) -> categories + "\n" + category);
            getLogger().info(menu);
        }
        return categoryMap;
    }

    @Override
    public Map<Integer, Product> getProductsByGenderAndCategory(Gender gender, Category category) {
        Map<Integer, Product> productMap = new LinkedHashMap<>();
        AtomicInteger number = new AtomicInteger(0);
        Connection connection = getConnection();
        try {
            PreparedStatement preparedStatement = connection.prepareStatement("select * from gap.products where gender=? and category =?");
            preparedStatement.setString(1, gender.getGender());
            preparedStatement.setString(2, category.toString());
            ResultSet resultSet = preparedStatement.executeQuery();
            ResultSetMetaData resultSetMetaData = resultSet.getMetaData();
            while (resultSet.next()) {
                Product product = new Product(resultSet.getString(1), resultSet.getString(2), Category.valueOf(resultSet.getString(3)), Color.valueOf(resultSet.getString(4)), resultSet.getDouble(5), Gender.valueOf(resultSet.getString(6).toUpperCase()), getProductSizes(resultSet.getInt(7), resultSet.getInt(8), resultSet.getInt(9), resultSet.getInt(10), resultSet.getInt(11)));
                productMap.put(number.incrementAndGet(), product);
            }
        } catch (SQLException e) {
            getLogger().warning(e.getMessage());
            getLogger().warning("\n-----------------------------\ngot error while fetching the gender. please try again\n-----------------------------\n");
            return getProductsByGenderAndCategory(gender, category);
        }
        if (productMap.isEmpty()) {
            getLogger().info("\n-----------------------------\nProducts not found in " + category + "\n-----------------------------\n");
        } else {
            String menu = productMap.entrySet().stream().map(entry -> entry.getKey() + ". " + entry.getValue().productShowcaseInMenu()).reduce("", (products, product) -> products + "\n" + product);
            getLogger().info(menu);
        }
        return productMap;
    }

    @Override
    public Map<Integer, Color> getColorsByGenderAndCategory(Gender gender, Category category) {
        Map<Integer, Color> colorMap = new LinkedHashMap<>();
        AtomicInteger number = new AtomicInteger(0);
        Connection connection = getConnection();
        try {
            PreparedStatement preparedStatement = connection.prepareStatement("select color from gap.products where gender=? and category=? group by color");
            preparedStatement.setString(1, gender.getGender());
            preparedStatement.setString(2, category.toString());
            ResultSet resultSet = preparedStatement.executeQuery();
            while (resultSet.next()) {
                colorMap.put(number.incrementAndGet(), Color.valueOf(resultSet.getString(1)));
            }
        } catch (SQLException e) {
            getLogger().warning(e.getMessage());
            getLogger().warning("\n-----------------------------\ngot error while fetching the gender. please try again\n-----------------------------\n");
            return getColorsByGenderAndCategory(gender, category);
        }
        if (colorMap.isEmpty()) {
            getLogger().info("\n-----------------------------\nColors not found in " + category + " for " + gender + "\n-----------------------------\n");
        } else {
            String menu = colorMap.entrySet().stream().map(entry -> entry.getKey() + ". " + entry.getValue()).reduce("", (colors, color) -> colors + "\n" + color);
            getLogger().info(menu);
        }
        return colorMap;
    }

    @Override
    public void showOrders() {
        boolean orderSelectFlag = true;
        Connection connection = getConnection();
        while (orderSelectFlag) {
            Map<Integer, Order> orders = getOrders(connection);
            String orderMenu = orders.entrySet().stream().map(entry -> entry.getKey() + ". " + entry.getValue().getOrderId() + "\t" + entry.getValue().getCustomerName()).reduce((allorders, order) -> allorders + "\n" + order).orElse("\n----------\nNo orders currently in your orders list");
            getLogger().info(orderMenu);
            getLogger().info("\n-----------------------------\nEnter 0 to go back.\n-----------------------------\n");
            getLogger().info("\n-----------------------------\nSelect order from the above list\n-----------------------------\n");
            int orderSelected = getInt();
            if (orderSelected == 0) {
                orderSelectFlag = false;
                break;
            }
            Order order = orders.get(orderSelected);
            if (order == null) {
                getLogger().info("\n-----------------------------\nPlease select an order available in the list\n-----------------------------\n");
                showOrders();
            } else {
                getLogger().info(order.showOrder());
                getLogger().info("\n-----------------------------\nenter 1 to view products \n2 to see orders \nenter any keyword to exit");
                int chooseProductList = getInt();
                switch (chooseProductList) {
                    case 1:
                        getLogger().info("\n-----------------------------\n" + order.showProducts() + "\n-----------------------------\n");
                        break;
                    case 2:
                        continue;
                    default:
                        orderSelectFlag = false;
                }
            }
        }
    }

    public Map<Integer, Order> getOrders(Connection connection) {
        Map<Integer, Order> ordersMenu = new LinkedHashMap<>();
        List<OrderedProduct> orderedProductList;
        AtomicInteger number = new AtomicInteger(0);
        try {
            PreparedStatement preparedStatement = connection.prepareStatement("select * from orders order by date desc");
            ResultSet resultSet = preparedStatement.executeQuery();
            while (resultSet.next()) {
                orderedProductList = new ArrayList<>();
                PreparedStatement innerPreparedStatement = connection.prepareStatement("select * from ordered_products where order_id = ?");
                innerPreparedStatement.setString(1, resultSet.getString(1));
                ResultSet innerResultSet = innerPreparedStatement.executeQuery();
                while (innerResultSet.next()) {
                    orderedProductList.add(new OrderedProduct(innerResultSet.getString(1), innerResultSet.getString(2), Category.valueOf(innerResultSet.getString(3)), Color.valueOf(innerResultSet.getString(4)), innerResultSet.getDouble(5), Gender.valueOf(innerResultSet.getString(6).toUpperCase()), Size.valueOf(innerResultSet.getString(7)), innerResultSet.getInt(8)));
                }
                ordersMenu.put(number.incrementAndGet(), new Order(resultSet.getString(1), resultSet.getDate(4).toLocalDate(), resultSet.getTime(5).toLocalTime(), resultSet.getString(2), resultSet.getString(3), resultSet.getDouble(6), orderedProductList));
            }
        } catch (Exception e) {
            getLogger().warning(e.getMessage());
        }
        return ordersMenu;
    }
}