package com.retail_store.order_management.utils;

import com.retail_store.order_management.application.OrderManagement;
import com.retail_store.order_management.constants.Category;
import com.retail_store.order_management.constants.Color;
import com.retail_store.order_management.constants.Gender;
import com.retail_store.order_management.constants.Size;
import com.retail_store.order_management.model.Order;
import com.retail_store.order_management.service.RetailStore;

import java.util.*;
import java.util.logging.Logger;

/**
 * Class Utility
 * This is class is used to finalize the all the methods present and those are static
 * Class used to import the values of the option given in the console and print accordingly
 * of the required choice chosen
 */
public final class Utility {
    private static final Scanner SCANNER = null;
    private static final Logger LOGGER = null;

    /**
     * No argument constructor
     */
    private Utility() {
    }

    /**
     * This static method is used to get the category list according to check with given inout message
     *
     * @param check
     * @param message
     * @return category
     */
    public static Category getCategory(boolean check, String message) {
        Category categoryValue;
        String categoryOptions = Arrays.stream(Category.values()).filter(category -> !category.equals(Category.EXIT)).map(category -> "(" + category.getOption() + ")" + category).reduce((categories, category) -> categories + " " + category).get() + "\nPress 0 to go back.";
        if (check) {
            getLogger().info("\n------------------------\n" + message + "\n------------------------\n" + categoryOptions + "\n------------------------\n");
        }
        int option = getInt();
        categoryValue = Arrays.stream(Category.values()).filter(category -> category.getOption() == option).findFirst().orElseGet(() -> {
            getLogger().info("\n------------------------\n+Please enter valid category\n------------------------\n" + categoryOptions + "\n------------------------\n");
            return getCategory(false, message);
        });
        return categoryValue.equals(Category.EXIT) ? null : categoryValue;
    }

    /**
     * This static method is used to get the Sizes of the given input option of the message
     *
     * @param check
     * @param message
     * @return
     */
    public static Size getSize(boolean check, String message) {
        Size sizeValue;
        String categoryOptions = Arrays.stream(Size.values()).filter(size -> !size.equals(Size.EXIT)).map(size -> "(" + size.getOption() + ")" + size).reduce((sizes, size) -> sizes + " " + size).get() + "\nPress 0 to go back.";
        if (check) {
            getLogger().info("\n------------------------\n" + message + "\n------------------------\n" + categoryOptions + "\n------------------------\n");
        }
        int option = getInt();
        sizeValue = Arrays.stream(Size.values()).filter(size -> size.getOption() == option).findFirst().orElseGet(() -> {
            getLogger().info("\n------------------------\nPlease enter valid category\n------------------------\n" + categoryOptions + "\n------------------------\n");
            return getSize(false, message);
        });
        return sizeValue.equals(Size.EXIT) ? null : sizeValue;
    }

    /**
     * This method is used to get the gender details of the given input option
     *
     * @param check
     * @param message
     * @return gender
     */
    public static Gender getGender(boolean check, String message) {
        Gender genderValue;
        String genderOptions = Arrays.stream(Gender.values()).filter(gender -> !gender.equals(Gender.EXIT)).map(gender -> "(" + gender.getOption() + ")" + gender).reduce((genders, gender) -> genders + " " + gender).get() + "\nPress 0 to go back.";
        if (check) {
            getLogger().info("\n------------------------\n" + message + "\n------------------------\n" + genderOptions + "\n------------------------\n");
        }
        int option = getInt();
        genderValue = Arrays.stream(Gender.values()).filter(gender -> gender.getOption() == option).findFirst().orElseGet(() -> {
            getLogger().info("\n------------------------\nPlease enter valid gender\n------------------------\n" + genderOptions + "\n------------------------\n");
            return getGender(false, message);
        });
        return genderValue.equals(Gender.EXIT) ? null : genderValue;
    }

    /**
     * This method is used to get all the colors of the given input option
     *
     * @param check
     * @param message
     * @return color
     */
    public static Color getColor(boolean check, String message) {
        Color colorValue;
        String colorOptions = Arrays.stream(Color.values()).filter(color -> !color.equals(Color.EXIT)).map(color -> "(" + color.getOption() + ")" + color).reduce((categories, category) -> categories + " " + category).get() + "\nPress 0 to go back.";
        if (check) {
            getLogger().info("\n------------------------\n" + message + "\n------------------------\n" + colorOptions + "\n------------------------\n");
        }
        int option = getInt();
        colorValue = Arrays.stream(Color.values()).filter(color -> color.getOption() == option).findFirst().orElseGet(() -> {
            getLogger().info("\n------------------------\n+Please enter valid color\n------------------------\n" + colorOptions + "\n------------------------\n");
            return getColor(false, message);
        });
        return colorValue.equals(Color.EXIT) ? null : colorValue;
    }

    /**
     * This method is used to get the message as per invalid option if
     * you enter other options or other input than shown there.
     *
     * @param flag
     * @param message
     * @return double
     */
    public static double getDouble(boolean flag, String message) {
        double number;
        Scanner scanner = new Scanner(System.in);
        if (flag) {
            getLogger().info(message);
        }
        try {
            number = scanner.nextDouble();
            if (number < 0) {
                throw new RuntimeException();
            }
            return number;
        } catch (InputMismatchException e) {
            getLogger().warning("\n------------------------\nPlease enter valid price\n------------------------\n");
            scanner.reset();
            return getDouble(false, message);
        }
    }

    /**
     * This method is used to get the message as per invalid option if
     * you enter other options or other input than shown there.
     *
     * @return int
     */
    public static int getInt() {
        int number;
        Scanner scanner = new Scanner(System.in);
        try {
            number = scanner.nextInt();
            if (number < 0) {
                throw new RuntimeException();
            }
            return number;
        } catch (Exception e) {
            getLogger().warning("\n------------------------\nPlease enter valid number\n------------------------\n");
            scanner.reset();
            return getInt();
        }
    }

    /**
     * This method is used to take phone number input
     * and validate it without any non numeric values and
     * numbers must be 10 and returns String phone number
     *
     * @return
     */
    public static String phoneNumberValidator() {
        String phoneNumber;
        try {
            phoneNumber = getScanner().next();
            if (phoneNumber == null || phoneNumber.isBlank() || phoneNumber.isEmpty() || phoneNumber.length() != 10) {
                throw new RuntimeException("Please enter valid number");
            }
        } catch (Exception e) {
            getLogger().warning("Please enter valid phone number");
            return phoneNumberValidator();
        }
        return phoneNumber.toString();
    }

    /**
     * This method is used to create a singleton
     * logger object that is used throughout the application
     *
     * @return
     */
    public static Logger getLogger() {
        return LOGGER == null ? Logger.getLogger(OrderManagement.class.getName()) : LOGGER;
    }

    /**
     * This method is used to create a singleton
     * scanner object that is used throughout the application
     *
     * @return
     */
    public static Scanner getScanner() {
        return SCANNER == null ? new Scanner(System.in) : SCANNER;
    }

    /**
     * This method is used to give a valid number that present in the stock range
     *
     * @return integer according to key type size
     */
    public static LinkedHashMap<Size, Integer> getProductSizes() {
        LinkedHashMap<Size, Integer> productSizes = new LinkedHashMap<>();
        getLogger().info("Please enter the available stock for particular sizes \n Number of products with size S");
        productSizes.put(Size.S, getInt());
        getLogger().info("Number of products with size M");
        productSizes.put(Size.M, getInt());
        getLogger().info("Number of products with size L");
        productSizes.put(Size.L, getInt());
        getLogger().info("Number of products with size XL");
        productSizes.put(Size.XL, getInt());
        getLogger().info("Number of products with size XXL");
        productSizes.put(Size.XXL, getInt());
        return productSizes;
    }

    /**
     * This method is used to give the product sizes available in the database according to
     * the key value pair inserted in the database
     *
     * @param s
     * @param m
     * @param l
     * @param xl
     * @param xxl
     * @return integer according to key type size
     */
    public static Map<Size, Integer> getProductSizes(int s, int m, int l, int xl, int xxl) {
        LinkedHashMap<Size, Integer> productSizes = new LinkedHashMap<>();
        productSizes.put(Size.S, s);
        productSizes.put(Size.M, m);
        productSizes.put(Size.L, l);
        productSizes.put(Size.XL, xl);
        productSizes.put(Size.XXL, xxl);
        return productSizes;
    }

    /**
     * This method is to generate automatic unique productId for the products
     * while adding them.
     *
     * @return string
     */
    public static String productIdGenerator() {
        return Arrays.stream(UUID.randomUUID().toString().split("-")).skip(1).findFirst().get() + "#" + RetailStore.class.getSimpleName();
    }

    /**
     * This method is to generate automatic unique orderId for the orders which are placed
     * by the end user.
     *
     * @return string
     */
    public static String orderIdGenerator() {
        return Arrays.stream(UUID.randomUUID().toString().split("-")).findFirst().get() + "#" + Order.class.getSimpleName();
    }
}